package com.googlecode.mycontainer.jms;

import java.util.ArrayList;
import java.util.Collection;

import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.naming.NamingException;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.broker.BrokerFactory;
import org.apache.activemq.broker.BrokerService;
import org.apache.activemq.command.ActiveMQDestination;
import org.apache.activemq.command.ActiveMQQueue;
import org.apache.activemq.command.ActiveMQTopic;

import com.googlecode.mycontainer.kernel.deploy.NamingDeployer;
import com.googlecode.mycontainer.kernel.naming.MyNameParser;

public class JMSDeployer extends NamingDeployer {

	private static final long serialVersionUID = 3150792465673952927L;

	private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory
			.getLogger(JMSDeployer.class);

	private BrokerService broker;

	private String uri = "broker:(vm://localhost)";

	private String connectionUri = "vm://localhost";

	private Collection<QueueDeployer> queueDeployers = new ArrayList<QueueDeployer>();

	private Collection<TopicDeployer> topicDeployers = new ArrayList<TopicDeployer>();

	public String getConnectionUri() {
		return connectionUri;
	}

	public void setConnectionUri(String connectionUri) {
		this.connectionUri = connectionUri;
	}

	public String getUri() {
		return uri;
	}

	public void setUri(String uri) {
		this.uri = uri;
	}

	public BrokerService getBroker() {
		if (broker == null) {
			try {
				broker = BrokerFactory.createBroker(uri);
				if (!uri.contains("useJmx")) {
					broker.setUseJmx(false);
				}
				if (!uri.contains("persistent")) {
					broker.setPersistent(false);
				}
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
		return broker;
	}

	@Override
	public void shutdown() {
		super.shutdown();
		try {
			if (broker != null) {
				LOG.info("Stopping JMS Server");
				broker.stop();
				LOG.info("Done");
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	protected Object getResource() {
		return new ActiveMQConnectionFactory(connectionUri);
		/*
		ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(
				connectionUri) {

			@Override
			protected ActiveMQConnection createActiveMQConnection(
					Transport transport, JMSStatsImpl stats) throws Exception {
				ActiveMQConnection ret = new ActiveMQConnection(transport,
						getClientIdGenerator(), stats) {
					public Session createSession(boolean transacted,
							int acknowledgeMode) throws JMSException {
						checkClosedOrFailed();
						ensureConnectionInfoSent();
						if (!transacted) {
							if (acknowledgeMode == Session.SESSION_TRANSACTED) {
								throw new JMSException(
										"acknowledgeMode SESSION_TRANSACTED cannot be used for an non-transacted Session");
							} else if (acknowledgeMode < Session.SESSION_TRANSACTED
									|| acknowledgeMode > 4) {
								throw new JMSException(
										"invalid acknowledgeMode: "
												+ acknowledgeMode
												+ ". Valid values are Session.AUTO_ACKNOWLEDGE (1), "
												+ "Session.CLIENT_ACKNOWLEDGE (2), Session.DUPS_OK_ACKNOWLEDGE (3), ActiveMQSession.INDIVIDUAL_ACKNOWLEDGE (4) or for transacted sessions Session.SESSION_TRANSACTED (0)");
							}

						}
						return new ActiveMQSession(
								this,
								getNextSessionId(),
								transacted ? Session.SESSION_TRANSACTED
										: (acknowledgeMode == Session.SESSION_TRANSACTED ? Session.AUTO_ACKNOWLEDGE
												: acknowledgeMode),
								isDispatchAsync(), isAlwaysSessionAsync()) {
							public MessageConsumer createConsumer(
									Destination destination,
									String messageSelector, boolean noLocal,
									MessageListener messageListener)
									throws JMSException {
								checkClosed();
								if (destination instanceof CustomDestination) {
									CustomDestination customDestination = (CustomDestination) destination;
									return customDestination.createConsumer(
											this, messageSelector, noLocal);
								}
								ActiveMQPrefetchPolicy prefetchPolicy = connection
										.getPrefetchPolicy();
								int prefetch = 0;
								if (destination instanceof Topic) {
									prefetch = prefetchPolicy
											.getTopicPrefetch();
								} else {
									prefetch = prefetchPolicy
											.getQueuePrefetch();
								}
								ActiveMQDestination activemqDestination = ActiveMQMessageTransformation
										.transformDestination(destination);
								return new ActiveMQMessageConsumer(
										this,
										getNextConsumerId(),
										activemqDestination,
										null,
										messageSelector,
										prefetch,
										prefetchPolicy
												.getMaximumPendingMessageLimit(),
										noLocal, false, isAsyncDispatch(),
										messageListener) {
									@Override
									public boolean iterate() {
										try {
											MessageListener listener = getMessageListener();
											if (listener != null) {
												Field f = ActiveMQMessageConsumer.class
														.getDeclaredField("unconsumedMessages");
												f.setAccessible(true);
												MessageDispatchChannel a = (MessageDispatchChannel) f
														.get(this);
												MessageDispatch md = a
														.dequeueNoWait();
												if (md != null) {
													dispatch(md);
													return true;
												}
											}
											return false;
										} catch (SecurityException e) {
											throw new RuntimeException(e);
										} catch (IllegalArgumentException e) {
											throw new RuntimeException(e);
										} catch (JMSException e) {
											throw new RuntimeException(e);
										} catch (NoSuchFieldException e) {
											throw new RuntimeException(e);
										} catch (IllegalAccessException e) {
											throw new RuntimeException(e);
										}
									}
								};
							}
						};

					}
				};
				return ret;
		};
		return connectionFactory;
				*/
	}

	public void createQueue(String queueName) {
		ActiveMQQueue queue = new ActiveMQQueue(queueName);

		QueueDeployer queueDeployer = new QueueDeployer(queue);
		queueDeployer.setContext(getContext());

		queueDeployers.add(queueDeployer);
	}

	public void createTopic(String topicName) {
		ActiveMQTopic topic = new ActiveMQTopic(topicName);

		TopicDeployer topicDeployer = new TopicDeployer(topic);
		topicDeployer.setContext(getContext());

		topicDeployers.add(topicDeployer);
	}

	@Override
	public void deploy() {
		try {
			setName(MyNameParser.parseClassName("resource",
					ConnectionFactory.class));
			super.deploy();

			Collection<ActiveMQDestination> q = deployQueues();
			Collection<ActiveMQDestination> t = deployTopics();
			if (uri != null) {
				Collection<ActiveMQDestination> all = new ArrayList<ActiveMQDestination>();
				all.addAll(q);
				all.addAll(t);
				ActiveMQDestination[] a = all
						.toArray(new ActiveMQDestination[0]);
				getBroker().setDestinations(a);

				getBroker().start();
			}
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (JMSException e) {
			throw new RuntimeException(e);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	private Collection<ActiveMQDestination> deployQueues() {
		Collection<ActiveMQDestination> queues = new ArrayList<ActiveMQDestination>();

		for (QueueDeployer deployer : queueDeployers) {
			deployer.deploy();
			queues.add((ActiveMQDestination) deployer.getQueue());
		}

		return queues;
	}

	private Collection<ActiveMQDestination> deployTopics() {
		Collection<ActiveMQDestination> topics = new ArrayList<ActiveMQDestination>();

		for (TopicDeployer deployer : topicDeployers) {
			deployer.deploy();
			topics.add((ActiveMQDestination) deployer.getTopic());
		}

		return topics;
	}
}
